/**
  @page WWDG_Example Window Watchdog example

  @verbatim
  ******************************************************************************
  * @file    WWDG/WWDG_Example/readme.txt
  * @author  MCD Application Team
  * @brief   Description of the Window Watchdog example.
  ******************************************************************************
  *
  * Copyright (c) 2019 STMicroelectronics. All rights reserved.
  *
  * This software component is licensed by ST under BSD 3-Clause license,
  * the "License"; You may not use this file except in compliance with the
  * License. You may obtain a copy of the License at:
  *                       opensource.org/licenses/BSD-3-Clause
  *
  ******************************************************************************
  @endverbatim

@par Example Description

Configuration of the HAL API to periodically update the WWDG counter and simulate a software fault that
generates an MCU WWDG reset when a predefined time period has elapsed.

At the beginning of the main program the HAL_Init() function is called to reset
all the peripherals, initialize the Flash interface and the systick.
Then the SystemClock_Config() function is used to configure the system
clock (SYSCLK) to run at 64 MHz.

The WWDG peripheral configuration is ensured by the HAL_WWDG_Init() function.
This later is calling the HAL_WWDG_MspInit()function which core is implementing
the configuration of the needed WWDG resources according to the used hardware (CLOCK
and NVIC). You may update this function to change WWDG configuration.

The WWDG timeout is set, through counter value, to 65,53 ms.
The refresh window is set in order to make user wait 24,58 ms after a wadchdog refresh,
before writing again counter. Hence the WWDG counter is refreshed each (24,58 + 1) ms in the
main program infinite loop to prevent a WWDG reset.
LED5 is toggling at same frequency, indicating that the program is running.

An EXTI Line is connected to a GPIO pin, and configured to generate an interrupt
on the rising edge of the signal.

The EXTI Line is used to simulate a software failure: once the EXTI Line event
occurs by pressing the Joystick Selection push-button (PC.00), the corresponding interrupt is served.

In the ISR, a write to invalid address generates a Hardfault exception containing
an infinite loop and preventing to return to main program (the WWDG counter is
not refreshed).
As a result, when the WWDG counter falls to 0x3F, WWDG reset occurs.

If the WWDG reset is generated, after the system resumes from reset, LED4 is turned ON for 4 seconds.

If the EXTI Line event does not occur, the WWDG counter is indefinitely refreshed
in the main program infinite loop, and there is no WWDG reset.

LED6 is turned ON and remains ON if any error occurs.

@note This example must be tested in standalone mode (not in debug).


@note Care must be taken when using HAL_Delay(), this function provides accurate
      delay (in milliseconds) based on variable incremented in SysTick ISR. This
      implies that if HAL_Delay() is called from a peripheral ISR process, then
      the SysTick interrupt must have higher priority (numerically lower)
      than the peripheral interrupt. Otherwise the caller ISR process will be blocked.
      To change the SysTick interrupt priority you have to use HAL_NVIC_SetPriority() function.

@note The application needs to ensure that the SysTick time base is always set to 1 millisecond
      to have correct HAL operation.


@par Keywords

System, WWDG, EXTI, update counter, MCU Reset, Timeout, Software fault

@par Directory contents

  - WWDG/WWDG_Example/Inc/stm32g0xx_hal_conf.h    HAL configuration file
  - WWDG/WWDG_Example/Inc/stm32g0xx_it.h          Interrupt handlers header file
  - WWDG/WWDG_Example/Inc/main.h                        Header for main.c module
  - WWDG/WWDG_Example/Src/stm32g0xx_it.c          Interrupt handlers
  - WWDG/WWDG_Example/Src/main.c                        Main program
  - WWDG/WWDG_Example/Src/stm32g0xx_hal_msp.c     HAL MSP file
  - WWDG/WWDG_Example/Src/system_stm32g0xx.c      STM32G0xx system source file


@par Hardware and Software environment

  - This example runs on STM32G071RBTx devices.

  - This example has been tested with STM32G071B-DISCO board and can be
    easily tailored to any other supported device and development board.


@par How to use it ?

In order to make the program work, you must do the following :
 - Open your preferred toolchain
 - Rebuild all files and load your image into target memory
 - Run the example


 * <h3><center>&copy; COPYRIGHT STMicroelectronics</center></h3>
 */

